import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.Timer;
import java.awt.event.*;
public class DrawningShip {
    private IDrawBlocks drawingBlocks;

    protected EntityShip entityShip;
    public EntityShip getEntityShip(){return entityShip;}
    /// <summary>
    /// Ширина окна
    /// </summary>
    protected int _pictureWidth;
    /// <summary>
    /// Высота окна
    /// </summary>
    protected int _pictureHeight;
    /// <summary>
    /// Левая координата прорисовки локомотива
    /// </summary>
    protected int startPosX;
    /// <summary>
    /// Верхняя кооридната прорисовки локомотива
    /// </summary>
    protected int startPosY;
    /// <summary>
    /// Координата X объекта
    /// </summary>
    public int GetPosX (){return  startPosX;}
    /// <summary>
    /// Координата Y объекта
    /// </summary>
    public int GetPosY (){return startPosY;}
    /// <summary>
    /// Ширина объекта
    /// </summary>
    public int GetWidth (){return  shipWidth;}
    /// <summary>
    /// Высота объекта
    /// </summary>
    public int GetHeight (){return  shipHeight;}
    /// <summary>
    /// Ширина прорисовки локомотива
    /// </summary>
    protected int shipWidth = 175;
    protected int shipHeight = 80;
    /// <summary>
    /// Инициализация свойств
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес</param>
    /// <param name="bodyColor">Цвет кузова</param>
    /// <param name="width">Ширина картинки</param>
    /// <param name="height">Высота картинки</param>
    public DrawningShip(int speed, double weight, Color bodyColor, int width, int height, int blocksType, int blocksNumber)
    {
        _pictureWidth = width;
        _pictureHeight = height;
        if (width < shipWidth || height < shipHeight)
            return;
        entityShip = new EntityShip(speed, weight, bodyColor);
        Random random = new Random();
        switch(blocksType){
            case 0:
                drawingBlocks = new DrawingBlocks();
                break;
//            case 1:
//                drawingBlocks = new DrawingCrossBlocks();
//                break;
//            case 2:
//                drawingBlocks = new DrawingRoundBlocks();
//                break;
            default:
                drawingBlocks = new DrawingBlocks();
                break;
        }
        drawingBlocks.setNumber(blocksNumber);
    }
    /// <summary>
    /// Установка позиции
    /// </summary>
    /// <param name="x">Координата X</param>
    /// <param name="y">Координата Y</param>
    public void SetPosition(int x, int y)
    {
        if (x < 0 || y < 0 || x + shipWidth > _pictureWidth || y + shipHeight > _pictureHeight) {
            x = 0;
            y = 0;
        }
        startPosX = x;
        startPosY = y;
    }

    /// <summary>
    /// Проверка, что объект может переместится по указанному направлению
    /// </summary>
    /// <param name="direction">Направление</param>
    /// <returns>true - можно переместится по указанному направлению</returns>
    public boolean CanMove(DirectionType direction)
    {
        if (entityShip  == null)
        {
            return false;
        }
        switch (direction)
        {
            case LEFT:
                return startPosX - entityShip .Step > 0;
            case RIGHT:
                return startPosX + shipWidth + entityShip .Step < _pictureWidth;
            case UP:
                return startPosY - entityShip .Step > 0;
            case DOWN:
                return startPosY + shipHeight + entityShip .Step < _pictureHeight;
            default:
                return false;
        }
    }
    public void MoveTransport(DirectionType direction)
    {
        if (!CanMove(direction) || entityShip == null)
        {
            return;
        }
        switch (direction)
        {
            //влево
            case LEFT:
                startPosX -= (int)entityShip.Step;
                break;
            //вверх
            case UP:
                startPosY -= (int)entityShip.Step;
                break;
            // вправо
            case RIGHT:
                startPosX += (int)entityShip.Step;
                break;
            //вниз
            case DOWN:
                startPosY += (int)entityShip.Step;
                break;
        }
    }
    public void drawTransport(Graphics2D g2d) {
        if (entityShip == null) {
            return;
        }
        BasicStroke pen = new BasicStroke(2);
        g2d.setStroke(pen);
        Color bodyColor = entityShip.BodyColor;
        //основа
        int[] xPoints = {startPosX + 5, startPosX + 120, startPosX + 160, startPosX + 120, startPosX + 5};
        int[] yPoints = {startPosY + 0, startPosY + 0, startPosY + 35, startPosY + 70, startPosY + 70};
        g2d.setColor(bodyColor);
        g2d.fillPolygon(xPoints, yPoints, 5);
        g2d.setPaint(Color.BLACK);
        g2d.drawPolygon(xPoints, yPoints, 5);
        //блоки
        g2d.setPaint(Color.DARK_GRAY);
        g2d.fillRect(startPosX + 70, startPosY + 15, 20, 40);
        g2d.setPaint(Color.BLACK);
        g2d.drawRect(startPosX + 70, startPosY + 15, 20, 40);
        g2d.setPaint(Color.DARK_GRAY);
        g2d.fillRect(startPosX + 40, startPosY + 25, 30, 20);
        g2d.setPaint(Color.BLACK);
        g2d.drawRect(startPosX + 40, startPosY + 25, 30, 20);
        g2d.setPaint(Color.DARK_GRAY);
        g2d.fillOval(startPosX + 100, startPosY + 20, 30, 30);
        g2d.setPaint(Color.BLACK);
        g2d.drawOval(startPosX + 100, startPosY + 20, 30, 30);
        //для ускорения
        g2d.setPaint(Color.YELLOW);
        g2d.fillRect(startPosX + 0, startPosY + 10, 5, 20);
        g2d.fillRect(startPosX + 0, startPosY + 40, 5, 20);
        g2d.setPaint(Color.BLACK);
        g2d.drawRect(startPosX + 0, startPosY + 40, 5, 20);
        g2d.drawRect(startPosX + 0, startPosY + 10, 5, 20);
        //блоки
        if (drawingBlocks != null) {
            drawingBlocks.drawBlocks(g2d, startPosX, startPosY);
        }
    }
}
